@@ -21,9 +21,6 @@
 #include <string.h>
 #include <errno.h>
 #include <stdlib.h>
-#ifdef HAVE_ICONV
-# include <iconv.h>
-#endif
 
 /* Include dirent for various systems */
 #ifdef HAVE_DIRENT_H
@@ -38,7 +35,7 @@
 #define DEBUG
 
 #include "playlist.h"
-#include "main.h"
+#include "common.h"
 #include "interface.h"
 #include "decoder.h"
 #include "options.h"
@@ -49,11 +46,6 @@
 #define FILE_LIST_INIT_SIZE	64
 #define READ_LINE_INIT_SIZE	256
 
-#ifdef HAVE_ICONV
-static iconv_t tags_iconv_desc = (iconv_t)(-1);
-static iconv_t files_iconv_desc = (iconv_t)(-1);
-#endif
-
 /* Is the string an URL? */
 inline int is_url (const char *str)
 {
@@ -61,6 +53,18 @@ inline int is_url (const char *str)
 		|| !strncmp(str, "ftp://", sizeof("ftp://")-1);
 }
 
+/* Return 1 if the file is a directory, 0 if not, -1 on error. */
+int isdir (const char *file)
+{
+	struct stat file_stat;
+
+	if (stat(file, &file_stat) == -1) {
+		error ("Can't stat %s: %s", file, strerror(errno));
+		return -1;
+	}
+	return S_ISDIR(file_stat.st_mode) ? 1 : 0;
+}
+
 enum file_type file_type (const char *file)
 {
 	struct stat file_stat;
@@ -80,27 +84,6 @@ enum file_type file_type (const char *file)
 	return F_OTHER;
 }
 
-/* Get file name from a path. Returned memory is malloc()ed. */
-static char *get_file (const char *path, const int strip_ext)
-{
-	char *fname;
-	char *ext;
-	
-	assert (path != NULL);
-
-	fname = strrchr (path, '/');
-
-	if (fname)
-		fname = xstrdup (fname + 1);
-	else
-		fname = xstrdup (path);
-
-	if (strip_ext && (ext = ext_pos(fname)))
-		*(ext-1) = 0;
-
-	return fname;
-}
-
 /* Make a title from the file name for the item. If hide extension != 0, strip
  * the file name from extension. */
 void make_file_title (struct plist *plist, const int num,
@@ -112,10 +95,17 @@ void make_file_title (struct plist *plist, const int num,
 	assert (!plist_deleted(plist, num));
 
 	if (file_type(plist->items[num].file) != F_URL) {
-		char *fname;
+		char *file = xstrdup (plist->items[num].file);
 		
-		fname = get_file (plist->items[num].file, hide_extension);
-		plist_set_title_file (plist, num, fname);
+		if (hide_extension) {
+			char *dot = strrchr (file, '.');
+
+			if (dot)
+				*dot = 0;
+		}
+
+		plist_set_title_file (plist, num, file);
+		free (file);
 	}
 	else
 		plist_set_title_file (plist, num, plist->items[num].file);
@@ -128,19 +118,12 @@ void make_tags_title (struct plist *plist, const int num)
 	assert (num >= 0 && num < plist->num);
 	assert (!plist_deleted(plist, num));
 
-	update_file (&plist->items[num]);
-
 	if (file_type(plist->items[num].file) == F_URL)
 		make_file_title (plist, num, 0);
 	else if (!plist->items[num].title_tags) {
 		char *title;
 		assert (plist->items[num].file != NULL);
 
-		plist->items[num].tags = read_file_tags (
-				plist->items[num].file,
-				plist->items[num].tags,
-				TAGS_COMMENTS);
-		
 		if (plist->items[num].tags->title) {
 			title = build_title (plist->items[num].tags);
 			plist_set_title_tags (plist, num, title);
@@ -152,48 +135,16 @@ void make_tags_title (struct plist *plist, const int num)
 	}
 }
 
-/* Make titles for the playlist items from the file names. */
-void make_titles_file (struct plist *plist)
-{
-	int hide_extension = options_get_int ("HideFileExtension");
-	int i;
-
-	for (i = 0; i < plist->num; i++)
-		if (!plist_deleted(plist, i))
-			make_file_title (plist, i, hide_extension);
-}
-
-/* Make titles for the playlist items from the tags. */
-void make_titles_tags (struct plist *plist)
-{
-	int i;
-	int interrupt = 0;
-
-	for (i = 0; i < plist->num; i++) {
-
-		if (user_wants_interrupt()) {
-			interrupt = 1;
-			interface_error ("Reading tags interrupted");
-			break;
-		}
-		
-		if (!plist_deleted(plist, i) && !plist->items[i].title_tags)
-			make_tags_title (plist, i);
-	}
-
-	if (interrupt)
-		make_titles_file (plist);
-}
-
 /* Switch playlist titles to title_file */
 void switch_titles_file (struct plist *plist)
 {
 	int i;
-
-	make_titles_file (plist);
-
+	int hide_extension = options_get_int("HideFileExtension");
+		
 	for (i = 0; i < plist->num; i++)
 		if (!plist_deleted(plist, i)) {
+			if (!plist->items[i].title_file)
+				make_file_title (plist, i, hide_extension);
 			assert (plist->items[i].title_file != NULL);
 			plist->items[i].title = plist->items[i].title_file;
 		}
@@ -204,17 +155,18 @@ void switch_titles_tags (struct plist *plist)
 {
 	int i;
 
-	make_titles_tags (plist);
-	
 	for (i = 0; i < plist->num; i++)
 		if (!plist_deleted(plist, i)) {
-
 			if (plist->items[i].title_tags)
 				plist->items[i].title
 					= plist->items[i].title_tags;
-			else
+			else {
+				if (!plist->items[i].title_file)
+					make_file_title (plist, i,
+							options_get_int("HideFileExtension"));
 				plist->items[i].title
 					= plist->items[i].title_file;
+			}
 		}
 }
 
@@ -280,117 +232,6 @@ void resolve_path (char *buf, const int size, const char *file)
 		buf[--len] = 0;
 }
 
-#ifdef HAVE_ICONV
-/* Parse iconv description in format FROM:TO. Returns the descriptopn. Does
- * fatal() on error. */
-static iconv_t parse_iconv_desc (const char *desc)
-{
-	char conv_str[100];
-	char *from, *to;
-	iconv_t iconv_desc;
-
-	conv_str[sizeof(conv_str)-1] = 0;
-	strncpy (conv_str, desc, sizeof(conv_str));
-
-	if (conv_str[sizeof(conv_str)-1])
-		fatal ("iconv string too long!");
-
-	from = conv_str;
-	to = strchr (conv_str, ':');
-	assert (to != NULL);
-	*to = 0;
-	to++;
-
-	if ((iconv_desc = iconv_open(to, from)) == (iconv_t)(-1))
-		fatal ("Can't use specified Iconv value ('%s'): %s", desc,
-				strerror(errno));
-	
-	return iconv_desc;
-}
-#endif
-
-void iconv_init ()
-{
-#ifdef HAVE_ICONV
-	if (options_get_str("TagsIconv"))
-		tags_iconv_desc = parse_iconv_desc (
-				options_get_str("TagsIconv"));
-	if (options_get_str("FileNamesIconv"))
-		files_iconv_desc = parse_iconv_desc (
-				options_get_str("FileNamesIconv"));
-#endif
-}
-
-/* Return a malloc()ed string converted using iconv().
- * if for_file_name is not 0, uses the conversion defined for file names.
- * Does free(str). For NULL returns NULL. */
-char *iconv_str (char *str, const int for_file_name)
-{
-#ifdef HAVE_ICONV
-	char buf[512];
-	char *inbuf, *outbuf;
-	size_t inbytesleft, outbytesleft;
-	char *converted;
-	iconv_t iconv_desc = for_file_name ? files_iconv_desc : tags_iconv_desc;
-
-	if (!str)
-		return NULL;
-	if (iconv_desc == (iconv_t)(-1))
-		return str;
-
-	inbuf = str;
-	outbuf = buf;
-	inbytesleft = strlen(inbuf);
-	outbytesleft = sizeof(buf) - 1;
-
-	iconv (iconv_desc, NULL, NULL, NULL, NULL);
-	
-	while (inbytesleft) {
-		if (iconv(iconv_desc, &inbuf, &inbytesleft, &outbuf, &outbytesleft)
-				== (size_t)(-1)) {
-			if (errno == EILSEQ) {
-				inbuf++;
-				inbytesleft--;
-				if (!--outbytesleft) {
-					*outbuf = 0;
-					break;
-				}
-				*(outbuf++) = '#';
-			}
-			else if (errno == EINVAL) {
-				*(outbuf++) = '#';
-				*outbuf = 0;
-				break;
-			}
-			else if (errno == E2BIG) {
-				outbuf[sizeof(buf)-1] = 0;
-				break;
-			}
-		}
-	}
-
-	*outbuf = 0;
-	converted = xstrdup (buf);
-	free (str);
-	
-	return converted;
-#else
-	return str;
-#endif
-}
-
-void iconv_cleanup ()
-{
-#ifdef HAVE_ICONV
-	if (files_iconv_desc != (iconv_t)(-1)
-			&& iconv_close(files_iconv_desc) == -1)
-		logit ("iconv_close() failed: %s", strerror(errno));
-	if (tags_iconv_desc != (iconv_t)(-1)
-			&& iconv_close(tags_iconv_desc) == -1)
-		logit ("iconv_close() failed: %s", strerror(errno));
-#endif
-}
-
 /* Read selected tags for a file into tags structure (or create it if NULL).
  * If some tags are already present, don't read them.
  * If present_tags is NULL, allocate new tags. */
@@ -423,6 +264,13 @@ struct file_tags *read_file_tags (const char *file,
 	}
 
 	if (needed_tags) {
+
+		/* This makes sure that we don't cause a memory leak */
+		assert (!((needed_tags & TAGS_COMMENTS) &&
+					(tags->title
+					 || tags->artist
+					 || tags->album)));
+
 		df->info (file, tags, needed_tags);
 		tags->filled |= tags_sel;
 	}
@@ -532,8 +380,7 @@ int read_directory (const char *directory, struct file_list *dirs,
 
 /* Recursively add files from the directory to the playlist. 
  * Return 1 if OK (and even some errors), 0 if the user interrupted. */
-int read_directory_recurr (const char *directory, struct plist *plist,
-		const int check_duplicates)
+int read_directory_recurr (const char *directory, struct plist *plist)
 {
 	DIR *dir;
 	struct dirent *entry;
@@ -565,8 +412,8 @@ int read_directory_recurr (const char *directory, struct plist *plist,
 		}
 		type = file_type (file);
 		if (type == F_DIR)
-			read_directory_recurr(file, plist, check_duplicates);
-		else if (type == F_SOUND)
+			read_directory_recurr(file, plist);
+		else if (type == F_SOUND && plist_find_fname(plist, file) == -1)
 			plist_add (plist, file);
 	}
 
@@ -741,51 +588,3 @@ time_t get_mtime (const char *file)
 	
 	return (time_t)-1;
 }
-
-/* Update the item data if the file was modified. */
-void update_file (struct plist_item *item)
-{
-	time_t mtime;
-
-	assert (item != NULL);
-
-	if ((mtime = get_mtime(item->file)) != item->mtime && mtime != -1) {
-		debug ("File %s was modified, updating", item->file);
-
-		if (item->tags) {
-			int needed_tags = item->tags->filled;
-
-			tags_free (item->tags);
-			item->tags = read_file_tags (item->file, NULL,
-					needed_tags);
-		}
-
-		if (item->title_tags) {
-			int update_title;
-			
-			update_title = item->title == item->title_tags;
-			free (item->title_tags);
-
-			item->tags = read_file_tags (item->file,
-					item->tags, TAGS_COMMENTS);
-			if (item->tags->title)
-				item->title_tags = build_title (item->tags);
-			else {
-				if (!item->title_file)
-					item->title_file = get_file (item->file,
-							options_get_int(
-								"HideFileExtension"));
-				item->title_tags = NULL;
-			}
-
-			if (update_title) {
-				if (item->title_tags)
-					item->title = item->title_tags;
-				else
-					item->title = item->title_file;
-			}
-		}
-
-		item->mtime = mtime;
-	}
-}
