@@ -1,6 +1,7 @@
 #ifndef FILES_H
 #define FILES_H
 
+#include <stdio.h>
 #include "playlist.h"
 
 struct file_list
@@ -12,10 +13,7 @@ struct file_list
 
 int read_directory (const char *directory, struct file_list *dirs,
 		struct file_list *playlists, struct plist *plist);
-int read_directory_recurr (const char *directory, struct plist *plist,
-		const int check_duplicates);
-void make_titles_file (struct plist *plist);
-void make_titles_tags (struct plist *plist);
+int read_directory_recurr (const char *directory, struct plist *plist);
 void resolve_path (char *buf, const int size, const char *file);
 char *ext_pos (const char *file);
 void file_list_free (struct file_list *list);
@@ -23,9 +21,6 @@ struct file_list *file_list_new ();
 enum file_type file_type (const char *file);
 int is_url (const char *str);
 char *read_line (FILE *file);
-void iconv_init ();
-void iconv_cleanup ();
-char *iconv_str (char *str, const int for_file_name);
 char *find_match_dir (char *dir);
 int file_exists (const char *file);
 time_t get_mtime (const char *file);
@@ -36,5 +31,6 @@ void switch_titles_tags (struct plist *plist);
 void make_tags_title (struct plist *plist, const int num);
 void make_file_title (struct plist *plist, const int num,
 		const int hide_extension);
+int isdir (const char *file);
 
 #endif
