@@ -424,7 +424,9 @@ static char *get_curr_file ()
 
 static void update_mixer_value ()
 {
-	iface_set_mixer_value (get_mixer_value());
+	int val = get_mixer_value ();
+	
+	iface_set_mixer_value (val >= 0 ? val : 0);
 }
 
 static void update_mixer_name ()
@@ -1390,7 +1392,8 @@ void init_interface (const int sock, const int logging, char **args,
 	update_mixer_name ();
 
 	signal (SIGQUIT, sig_quit);
-	/*signal (SIGTERM, sig_quit);*/
+	signal (SIGTERM, sig_quit);
+	signal (SIGHUP, sig_quit);
 	signal (SIGINT, sig_interrupt);
 	
 #ifdef SIGWINCH
@@ -1471,12 +1474,15 @@ static void do_resize ()
 }
 #endif
 
-static void go_dir_up ()
+/* Strip the last directory from the path. Returned memory is mallod()ed. */
+static char *dir_up (const char *path)
 {
-	char *dir;
 	char *slash;
+	char *dir;
 
-	dir = xstrdup (cwd);
+	assert (path != NULL);
+
+	dir = xstrdup (path);
 	slash = strrchr (dir, '/');
 	assert (slash != NULL);
 	if (slash == dir)
@@ -1484,6 +1490,14 @@ static void go_dir_up ()
 	else
 		*slash = 0;
 
+	return dir;
+}
+
+static void go_dir_up ()
+{
+	char *dir;
+
+	dir = dir_up (cwd);
 	go_to_dir (dir, 0);
 	free (dir);
 }
@@ -1813,10 +1827,15 @@ static void cmd_clear_playlist ()
 static void go_to_music_dir ()
 {
 	if (options_get_str("MusicDir")) {
-		if (file_type(options_get_str("MusicDir")) == F_DIR)
-			go_to_dir (options_get_str("MusicDir"), 0);
-		else if (file_type(options_get_str("MusicDir")) == F_PLAYLIST)
-			go_to_playlist (options_get_str("MusicDir"));
+		char music_dir[PATH_MAX] = "/";
+
+		resolve_path (music_dir, sizeof(music_dir),
+				options_get_str("MusicDir"));
+		
+		if (file_type(music_dir) == F_DIR)
+			go_to_dir (music_dir, 0);
+		else if (file_type(music_dir) == F_PLAYLIST)
+			go_to_playlist (music_dir);
 		else
 			error ("MusicDir is neither a directory nor a "
 					"playlist.");
@@ -2064,6 +2083,12 @@ static void entry_key_search (const struct iface_key *k)
 		iface_entry_disable ();
 		
 		if (text[0]) {
+
+			if (!strcmp(file, "..")) {
+				free (file);
+				file = dir_up (cwd);
+			}
+			
 			if (is_url(file))
 				play_from_url (file);
 			else if (file_type(file) == F_DIR)
@@ -2842,6 +2867,41 @@ static void exec_custom_command (const char *option)
 		free (args[--arg_num]);
 	} while (arg_num);
 	free (cmd);
+
+	if (iface_in_dir_menu())
+		reread_dir ();
+}
+
+static void go_to_fast_dir (const int num)
+{
+	char option_name[20];
+
+	assert (num >= 1 && num <= 10);
+
+	sprintf (option_name, "FastDir%d", num);
+	
+	if (options_get_str(option_name)) {
+		char dir[PATH_MAX] = "/";
+		
+		resolve_path (dir, sizeof(dir), options_get_str(option_name));
+		go_to_dir (dir, 0);
+	}
+	else
+		error ("%s is not defined", option_name);
+}
+
+static void toggle_playlist_full_paths (void)
+{
+	int new_val = !options_get_int("PlaylistFullPaths");
+	
+	option_set_int ("PlaylistFullPaths", new_val);
+
+	if (new_val)
+		iface_set_status ("PlaylistFullPaths: on");
+	else
+		iface_set_status ("PlaylistFullPaths: off");
+
+	update_iface_menu (IFACE_MENU_PLIST, playlist);
 }
 
 /* Handle key */
@@ -2904,6 +2964,9 @@ static void menu_key (const struct iface_key *k)
 			case KEY_CMD_TOGGLE_MENU:
 				toggle_menu ();
 				break;
+			case KEY_CMD_TOGGLE_PLAYLIST_FULL_PATHS:
+				toggle_playlist_full_paths ();
+				break;
 			case KEY_CMD_PLIST_ADD_FILE:
 				add_file_plist ();
 				break;
@@ -3023,84 +3086,34 @@ static void menu_key (const struct iface_key *k)
 				set_mixer (90);
 				break;
 			case KEY_CMD_FAST_DIR_1:
-				if (options_get_str("FastDir1"))
-					go_to_dir (options_get_str(
-								"FastDir1"), 0);
-				else
-					error ("FastDir1 not "
-							"defined");
+				go_to_fast_dir (1);
 				break;
 			case KEY_CMD_FAST_DIR_2:
-				if (options_get_str("FastDir2"))
-					go_to_dir (options_get_str(
-								"FastDir2"), 0);
-				else
-					error ("FastDir2 not "
-							"defined");
+				go_to_fast_dir (2);
 				break;
 			case KEY_CMD_FAST_DIR_3:
-				if (options_get_str("FastDir3"))
-					go_to_dir (options_get_str(
-								"FastDir3"), 0);
-				else
-					error ("FastDir3 not "
-							"defined");
+				go_to_fast_dir (3);
 				break;
 			case KEY_CMD_FAST_DIR_4:
-				if (options_get_str("FastDir4"))
-					go_to_dir (options_get_str(
-								"FastDir4"), 0);
-				else
-					error ("FastDir4 not "
-							"defined");
+				go_to_fast_dir (4);
 				break;
 			case KEY_CMD_FAST_DIR_5:
-				if (options_get_str("FastDir5"))
-					go_to_dir (options_get_str(
-								"FastDir5"), 0);
-				else
-					error ("FastDir5 not "
-							"defined");
+				go_to_fast_dir (5);
 				break;
 			case KEY_CMD_FAST_DIR_6:
-				if (options_get_str("FastDir6"))
-					go_to_dir (options_get_str(
-								"FastDir6"), 0);
-				else
-					error ("FastDir6 not "
-							"defined");
+				go_to_fast_dir (6);
 				break;
 			case KEY_CMD_FAST_DIR_7:
-				if (options_get_str("FastDir7"))
-					go_to_dir (options_get_str(
-								"FastDir7"), 0);
-				else
-					error ("FastDir7 not "
-							"defined");
+				go_to_fast_dir (7);
 				break;
 			case KEY_CMD_FAST_DIR_8:
-				if (options_get_str("FastDir8"))
-					go_to_dir (options_get_str(
-								"FastDir8"), 0);
-				else
-					error ("FastDir8 not "
-							"defined");
+				go_to_fast_dir (8);
 				break;
 			case KEY_CMD_FAST_DIR_9:
-				if (options_get_str("FastDir9"))
-					go_to_dir (options_get_str(
-								"FastDir9"), 0);
-				else
-					error ("FastDir9 not "
-							"defined");
+				go_to_fast_dir (9);
 				break;
 			case KEY_CMD_FAST_DIR_10:
-				if (options_get_str("FastDir10"))
-					go_to_dir (options_get_str(
-								"FastDir10"), 0);
-				else
-					error ("FastDir10 not "
-							"defined");
+				go_to_fast_dir (10);
 				break;
 			case KEY_CMD_TOGGLE_MIXER:
 				debug ("Toggle mixer.");
@@ -3334,8 +3347,8 @@ void interface_cmdline_clear_plist (int server_sock)
 		send_int_to_srv (CMD_PLIST_SET_SERIAL);
 		send_int_to_srv (serial);
 		send_int_to_srv (CMD_LIST_CLEAR);
+		send_int_to_srv (CMD_UNLOCK);
 	}
-	send_int_to_srv (CMD_UNLOCK);
 	
 	unlink (create_file_name("playlist.m3u"));
 
