@@ -612,6 +612,29 @@ static void entry_add_text_to_history (struct entry *e)
 	free (text);
 }
 
+/* Return the list menu height inside the side menu. */
+static int side_menu_get_menu_height (const struct side_menu *m)
+{
+	if (m->posy + m->height == LINES - 4)
+		return m->height - 1;
+	return m->height - 2;
+}
+
+static void side_menu_init_menu (struct side_menu *m)
+{
+	int height;
+	
+	assert (m != NULL);
+
+	if (m->posy + m->height == LINES - 4)
+		height = m->height - 1;
+	else
+		height = m->height - 2;
+
+	m->menu.list.main = menu_new (m->win, m->posx + 1, m->posy + 1,
+			m->width - 2, side_menu_get_menu_height(m));
+}
+
 static void side_menu_init (struct side_menu *m, const enum side_menu_type type,
 		WINDOW *parent_win, const struct window_params *wp)
 {
@@ -634,8 +657,7 @@ static void side_menu_init (struct side_menu *m, const enum side_menu_type type,
 	m->total_time_for_all = 0;
 
 	if (type == MENU_DIR || type == MENU_PLAYLIST) {
-		m->menu.list.main = menu_new (m->win, m->posx + 1, m->posy + 1,
-				m->width - 2, m->height - 1);
+		side_menu_init_menu (m);
 		m->menu.list.copy = NULL;
 		
 		menu_set_items_numbering (m->menu.list.main,
@@ -649,8 +671,7 @@ static void side_menu_init (struct side_menu *m, const enum side_menu_type type,
 				get_color(CLR_MENU_ITEM_INFO));
 	}
 	else if (type == MENU_THEMES) {
-		m->menu.list.main = menu_new (m->win, m->posx + 1, m->posy + 1,
-				m->width - 2, m->height - 1);
+		side_menu_init_menu (m);
 		m->menu.list.copy = NULL;
 	}
 	else
@@ -872,15 +893,15 @@ void sec_to_min (char *buff, const int seconds)
 /* Make a title suitable to display in a menu from the title of a playlist item.
  * Returned memory is malloc()ed.
  * made_from tags - was the playlist title made from tags? 
- * for_plist - is this title to be displayed in the playlist?
+ * full_paths - If the title is the file name, use the full path?
  */
 static char *make_menu_title (const char *plist_title,
-		const int made_from_tags, const int for_plist)
+		const int made_from_tags, const int full_path)
 {
 	char *title = xstrdup (plist_title);
 	
 	if (!made_from_tags) {
-		if (!for_plist) {
+		if (!full_path) {
 
 			/* Use only the file name instead of the full path. */
 			char *slash = strrchr (title, '/');
@@ -898,11 +919,11 @@ static char *make_menu_title (const char *plist_title,
 }
 
 /* Add an item from the playlist to the menu.
- * If for_playlist has non-zero value, full paths will be displayed instead of
+ * If full_paths has non-zero value, full paths will be displayed instead of
  * just file names.
  * Return a non-zero value if the added item is visible on the screen. */
 static int add_to_menu (struct menu *menu, const struct plist *plist,
-		const int num, const int for_playlist)
+		const int num, const int full_paths)
 {
 	struct menu_item *added;
 	const struct plist_item *item = &plist->items[num];
@@ -910,7 +931,7 @@ static int add_to_menu (struct menu *menu, const struct plist *plist,
 	const char *type_name;
 
 	title = make_menu_title (item->title, item->title == item->title_tags,
-			for_playlist);
+			full_paths);
 	added = menu_add (menu, title, plist_file_type(plist, num), item->file);
 	free (title);
 
@@ -931,7 +952,7 @@ static int add_to_menu (struct menu *menu, const struct plist *plist,
 		type_name = "";
 	menu_item_set_format (added, type_name);
 
-	if (for_playlist && item->title == item->title_file)
+	if (full_paths && item->title == item->title_file)
 		menu_item_set_align (added, MENU_ALIGN_RIGHT);
 
 	return menu_is_visible (menu, added);
@@ -946,8 +967,7 @@ static void side_menu_clear (struct side_menu *m)
 	assert (m->menu.list.copy == NULL);
 
 	menu_free (m->menu.list.main);
-	m->menu.list.main = menu_new (m->win, m->posx + 1, m->posy + 1,
-			m->width - 2, m->height - 1);
+	side_menu_init_menu (m);
 	menu_set_items_numbering (m->menu.list.main, m->type == MENU_PLAYLIST
 			&& options_get_int("PlaylistNumbering"));
 	
@@ -1010,7 +1030,8 @@ static void side_menu_make_list_content (struct side_menu *m,
 	for (i = 0; i < files->num; i++) {
 		if (!plist_deleted(files, i))
 			add_to_menu (m->menu.list.main, files, i,
-					m->type == MENU_PLAYLIST);
+					m->type == MENU_PLAYLIST
+					&& options_get_int("PlaylistFullPaths"));
 	}
 
 	m->total_time = plist_total_time (files, &m->total_time_for_all);
@@ -1056,16 +1077,40 @@ static void side_menu_draw_frame (const struct side_menu *m)
 	
 	/* Border */
 	wattrset (m->win, get_color(CLR_FRAME));
+
+	/* upper left corner */
 	wmove (m->win, m->posy, m->posx);
 	waddch (m->win, lines.ulcorn);
+
+	/* upper line */
 	whline (m->win, lines.horiz, m->width - 2);
+	
+	/* upper right corner */
 	wmove (m->win, m->posy, m->posx + m->width - 1);
 	waddch (m->win, lines.urcorn);
+	
+	/* left line */
 	wmove (m->win, m->posy + 1, m->posx);
 	wvline (m->win, lines.vert, m->height - 1);
+	
+	/* right line */
 	wmove (m->win, m->posy + 1, m->posx + m->width - 1);
 	wvline (m->win, lines.vert, m->height - 1);
 
+	if (m->posy + m->height < LINES - 4) {
+
+		/* bottom left corner */
+		wmove (m->win, m->posy + m->height - 1, m->posx);
+		waddch (m->win, lines.llcorn);
+	
+		/* bottom line */
+		whline (m->win, lines.horiz, m->width - 2);
+
+		/* bottom right corner */
+		wmove (m->win, m->posy + m->height - 1, m->posx + m->width - 1);
+		waddch (m->win, lines.lrcorn);	
+	}
+
 	/* The title */
 	if (title) {
 		wmove (m->win, m->posy, m->posx + m->width / 2
@@ -1200,7 +1245,7 @@ static void side_menu_set_curr_item_title (struct side_menu *m,
 /* Update menu item using the playlist item. */
 static void update_menu_item (struct menu_item *mi,
 		const struct plist *plist,
-		const int n, const int for_plist)
+		const int n, const int full_path)
 {
 	char *title;
 	const struct plist_item *item;
@@ -1221,11 +1266,11 @@ static void update_menu_item (struct menu_item *mi,
 		menu_item_set_time (mi, "");
 
 	title = make_menu_title (item->title,
-			item->title == item->title_tags, for_plist);
+			item->title == item->title_tags, full_path);
 
 	menu_item_set_title (mi, title);
 	
-	if (for_plist && item->title == item->title_file)
+	if (full_path && item->title == item->title_file)
 		menu_item_set_align (mi, MENU_ALIGN_RIGHT);
 	else
 		menu_item_set_align (mi, MENU_ALIGN_LEFT);
@@ -1253,12 +1298,14 @@ static int side_menu_update_item (struct side_menu *m,
 	assert (file != NULL);
 
 	if ((mi = menu_find(m->menu.list.main, file))) {
-		update_menu_item (mi, plist, n, m->type == MENU_PLAYLIST);
+		update_menu_item (mi, plist, n, m->type == MENU_PLAYLIST
+				&& options_get_int("PlaylistFullpaths"));
 		visible = menu_is_visible (m->menu.list.main, mi);
 	}
 	if (m->menu.list.copy
 			&& (mi = menu_find(m->menu.list.copy, file))) {
-		update_menu_item (mi, plist, n, m->type == MENU_PLAYLIST);
+		update_menu_item (mi, plist, n, m->type == MENU_PLAYLIST
+				&& options_get_int("PlaylistFullpaths"));
 		visible = visible || menu_is_visible (m->menu.list.main, mi);
 	}
 
@@ -1317,7 +1364,9 @@ static int side_menu_add_plist_item (struct side_menu *m,
 
 	visible = add_to_menu (m->menu.list.copy ? m->menu.list.copy
 			: m->menu.list.main,
-			plist, num, m->type == MENU_PLAYLIST);
+			plist, num,
+			m->type == MENU_PLAYLIST
+			&& options_get_int("PlaylistFullPaths"));
 	m->total_time = plist_total_time (plist, &m->total_time_for_all);
 
 	return visible;
@@ -1491,11 +1540,11 @@ static void side_menu_resize (struct side_menu *m,
 	if (m->type == MENU_DIR || m->type == MENU_PLAYLIST
 			|| m->type == MENU_THEMES) {
 		menu_update_size (m->menu.list.main, m->posx + 1, m->posy + 1,
-				m->width - 2, m->height - 1);
+				m->width - 2, side_menu_get_menu_height(m));
 		if (m->menu.list.copy)
 			menu_update_size (m->menu.list.copy, m->posx + 1,
 					m->posy + 1, m->width - 2,
-					m->height - 1);
+					side_menu_get_menu_height(m));
 	}
 	else
 		abort ();
