@@ -1,6 +1,6 @@
 /*
  * MOC - music on console
- * Copyright (C) 2003-2005 Damian Pietras <daper@daper.net>
+ * Copyright (C) 2003 - 2005 Damian Pietras <daper@daper.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,12 +31,13 @@
 #include <strings.h>
 #include <pthread.h>
 #include <unistd.h>
+#include <assert.h>
 
 #include "server.h"
 #include "audio.h"
 #include "log.h"
 #include "options.h"
-#include "main.h"
+#include "common.h"
 
 static int mixer_fd = -1;
 static int volatile dsp_fd = -1;
@@ -81,18 +82,11 @@ static int set_capabilities (struct output_driver_caps *caps)
 		caps->formats |= SFMT_S8;
 	if (format_mask & AFMT_U8)
 		caps->formats |= SFMT_U8;
-	if (format_mask & AFMT_S16_NE)
-		caps->formats |= SFMT_S16;
-#ifdef AFMT_S32_LE
-	if (format_mask & AFMT_S32_LE)
-		caps->formats |= SFMT_S32;
-#endif
-#ifdef AFMT_U32_LE
-	if (format_mask & AFMT_U32_LE)
-		caps->formats |= SFMT_U32;
-#endif
-
-	caps->formats |= SFMT_LE;
+	
+	if (format_mask & AFMT_S16_LE)
+		caps->formats |= SFMT_S16 | SFMT_LE;
+	if (format_mask & AFMT_S16_BE)
+		caps->formats |= SFMT_S16 | SFMT_BE;
 
 	if (!caps->formats) {
 		error ("No known format supported by the audio device.");
@@ -236,18 +230,11 @@ static int oss_set_params ()
 			req_format = AFMT_U8;
 			break;
 		case SFMT_S16:
-			req_format = AFMT_S16_LE;
+			if (params.fmt & SFMT_LE)
+				req_format = AFMT_S16_LE;
+			else
+				req_format = AFMT_S16_BE;
 			break;
-#ifdef AFMT_S32_LE
-		case SFMT_S32:
-			req_format = AFMT_S32_LE;
-			break;
-#endif
-#ifdef AFMT_U32_LE
-		case SFMT_U32:
-			req_format = AFMT_U32_LE;
-			break;
-#endif
 		default:
 			error ("format %s is not supported by the device",
 				sfmt_str(params.fmt, fmt_name,
