@@ -20,7 +20,7 @@
 
 #define DEBUG
 
-#include "main.h"
+#include "common.h"
 #include "log.h"
 #include "decoder.h"
 #include "audio.h"
@@ -38,7 +38,9 @@ enum request
 {
 	REQ_NOTHING,
 	REQ_SEEK,
-	REQ_STOP
+	REQ_STOP,
+	REQ_PAUSE,
+	REQ_UNPAUSE
 };
 
 struct bitrate_list_node
@@ -801,6 +803,25 @@ void player_seek (const int sec)
 	UNLOCK (request_cond_mutex);
 }
 
+/* Stop playing, clear the output buffer, but allow to unpause by starting
+ * playing the same stream. This is usefull for internet streams that can't
+ * be really paused. */
+void player_pause ()
+{
+	request = REQ_PAUSE;
+	LOCK (request_cond_mutex);
+	pthread_cond_signal (&request_cond);
+	UNLOCK (request_cond_mutex);
+}
+
+void player_unpause ()
+{
+	request = REQ_UNPAUSE;
+	LOCK (request_cond_mutex);
+	pthread_cond_signal (&request_cond);
+	UNLOCK (request_cond_mutex);
+}
+
 /* Return tags for the currently played file or NULL if there are no tags.
  * Tags are duplicated. */
 struct file_tags *player_get_curr_tags ()
