@@ -1,6 +1,6 @@
 /*
  * MOC - music on console
- * Copyright (C) 2004 Damian Pietras <daper@daper.net>
+ * Copyright (C) 2004 - 2006 Damian Pietras <daper@daper.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 #include "playlist.h"
 #include "playlist_file.h"
 #include "log.h"
-#include "main.h"
+#include "common.h"
 #include "files.h"
 #include "options.h"
 #include "interface.h"
@@ -140,12 +140,10 @@ static int plist_load_m3u (struct plist *plist, const char *fname,
 			after_extinf = 1;
 			last_added = plist_add (plist, NULL);
 			plist_set_title_tags (plist, last_added, comma + 1);
-			plist->items[last_added].tags = tags_new ();
-			if (*time_text) {
-				plist->items[last_added].tags->time = time_sec;
-				plist->items[last_added].tags->filled
-					|= TAGS_TIME;
-			}
+			
+			if (*time_text)
+				plist_set_item_time (plist, last_added,
+						time_sec);
 		}
 		else if (line[0] != '#') {
 			char path[2*PATH_MAX];
@@ -420,14 +418,20 @@ static int plist_save_m3u (struct plist *plist, const char *fname,
 	
 	for (i = 0; i < plist->num; i++)
 		if (!plist_deleted(plist, i)) {
+			int ret;
 
 			/* EXTM3U */
-			if (fprintf(file, "#EXTINF:%d,%s\r\n",
+			if (plist->items[i].tags)
+				ret = fprintf (file, "#EXTINF:%d,%s\r\n",
 						plist->items[i].tags->time,
 						plist->items[i].title_tags ?
 						plist->items[i].title_tags
-						: plist->items[i].title_file)
-					< 0) {
+						: plist->items[i].title_file);
+			else
+				ret = fprintf (file, "#EXTINF:%d,%s\r\n", 0,
+						plist->items[i].title_file);
+			
+			if (ret < 0) {
 				error ("Error writing playlist: %s",
 						strerror(errno));
 				fclose (file);
@@ -502,7 +506,6 @@ static void find_common_path (char *buf, const int buf_size,
 int plist_save (struct plist *plist, const char *file, const char *cwd)
 {
 	char common_path[PATH_MAX+1];
-	int i;
 
 	if (cwd)
 		
@@ -510,26 +513,6 @@ int plist_save (struct plist *plist, const char *file, const char *cwd)
 		 * relative paths) */
 		find_common_path (common_path, sizeof(common_path), plist);
 
-	make_titles_tags (plist);
-
-	if (user_wants_interrupt()) {
-		error ("Saving the playlist aborted");
-		return 0;
-	}
-
-	/* Get times */
-	for (i = 0; i < plist->num; i++)
-		if (!plist_deleted(plist, i)) {
-			if (user_wants_interrupt()) {
-				error ("Saving the playlist aborted");
-				return 0;
-			}
-			plist->items[i].tags = read_file_tags (
-					plist->items[i].file,
-					plist->items[i].tags,
-					TAGS_TIME);
-		}
-
 	/* FIXME: checkif it possible to just add some directories to make
 	 * relative path working. */
 	return plist_save_m3u (plist, file, cwd && !strcmp(common_path, cwd) ?
